@@ -86,7 +86,7 @@ int sys_isabsolutepath(const char *dir)
 }
 
 /* expand env vars and ~ at the beginning of a path and make a copy to return */
-static void sys_expandpath(const char *from, char *to, int bufsize)
+void sys_expandpath(const char *from, char *to, int bufsize)
 {
     if ((strlen(from) == 1 && from[0] == '~') || (strncmp(from,"~/", 2) == 0))
     {
