@@ -11,9 +11,73 @@
 #include "s_stuff.h"
 #include "m_private_utils.h"
 
+#ifdef _WIN32
+#ifndef PD_FWPRINTF_NARROW_FORMATTER
+#if __USE_MINGW_ANSI_STDIO
+    /* This is a workaround for a bug in the old msvcrt.dll used by MinGW */
+    #define PD_FWPRINTF_NARROW_FORMATTER L"%s"
+#else
+    /* Covers modern C runtimes on MSYS2 & MSVC */
+    #define PD_FWPRINTF_NARROW_FORMATTER L"%S"
+#endif
+#endif /* PD_FWPRINTF_NARROW_FORMATTER */
+#endif /* _WIN32 */
+
 t_printhook sys_printhook = NULL;
 int sys_printtostderr;
 
+#ifdef _WIN32
+
+    /* NB: Unlike vsnprintf(), _vsnprintf() does *not* null-terminate
+    the output if the resulting string is too large to fit into the buffer.
+    Also, it just returns -1 instead of the required number of bytes.
+    Strictly speaking, the UCRT in Windows 10 actually contains a standard-
+    conforming vsnprintf() function that is not just an alias for _vsnprintf().
+    However, MinGW traditionally links against the old msvcrt.dll runtime library.
+    Recent versions of MinGW seem to have their own (standard-conformating)
+    implementation of vsnprintf(), but to ensure portability we rather use our
+    own implementation for all Windows builds. */
+int pd_vsnprintf(char *buf, size_t size, const char *fmt, va_list argptr)
+{
+    int ret = _vsnprintf(buf, size, fmt, argptr);
+    if (ret < 0)
+    {
+            /* null-terminate the buffer and get the required number of bytes. */
+        ret = _vscprintf(fmt, argptr);
+        buf[size - 1] = '\0';
+    }
+    return ret;
+}
+
+int pd_snprintf(char *buf, size_t size, const char *fmt, ...)
+{
+    int ret;
+    va_list ap;
+    va_start(ap, fmt);
+    ret = pd_vsnprintf(buf, size, fmt, ap);
+    va_end(ap);
+    return ret;
+}
+
+#else
+
+int pd_vsnprintf(char *buf, size_t size, const char *fmt, va_list argptr)
+{
+    return vsnprintf(buf, size, fmt, argptr);
+}
+
+int pd_snprintf(char *buf, size_t size, const char *fmt, ...)
+{
+    int ret;
+    va_list ap;
+    va_start(ap, fmt);
+    ret = vsnprintf(buf, size, fmt, ap);
+    va_end(ap);
+    return ret;
+}
+
+#endif
+
 /* escape characters for tcl/tk */
 char* pdgui_strnescape(char *dst, size_t dstlen, const char *src, size_t srclen)
 {
@@ -49,14 +113,10 @@ static void dopost(const char *s)
 {
     if (STUFF->st_printhook)
         (*STUFF->st_printhook)(s);
-    else if (sys_printtostderr || !sys_havegui())
+    else if (sys_printtostderr || !sys_havetkproc())
     {
 #ifdef _WIN32
-    #ifdef _MSC_VER
-        fwprintf(stderr, L"%S", s);
-    #else
-        fwprintf(stderr, L"%s", s);
-    #endif
+        fwprintf(stderr, PD_FWPRINTF_NARROW_FORMATTER, s);
         fflush(stderr);
 #else
         fprintf(stderr, "%s", s);
@@ -76,17 +136,13 @@ static void doerror(const void *object, const char *s)
     // what about sys_printhook_error ?
     if (STUFF->st_printhook)
     {
-        snprintf(upbuf, MAXPDSTRING-1, "error: %s", s);
+        pd_snprintf(upbuf, MAXPDSTRING-1, "error: %s", s);
         (*STUFF->st_printhook)(upbuf);
     }
-    else if (sys_printtostderr)
+    else if (sys_printtostderr || !sys_havetkproc())
     {
 #ifdef _WIN32
-    #ifdef _MSC_VER
-        fwprintf(stderr, L"error: %S", s);
-    #else
-        fwprintf(stderr, L"error: %s", s);
-    #endif
+        fwprintf(stderr, L"error: " PD_FWPRINTF_NARROW_FORMATTER, s);
         fflush(stderr);
 #else
         fprintf(stderr, "error: %s", s);
@@ -94,7 +150,7 @@ static void doerror(const void *object, const char *s)
     }
     else
         pdgui_vmess("::pdwindow::logpost", "ois",
-                  object, 1, s);
+                  object, PD_ERROR, s);
 }
 
 static void dologpost(const void *object, const int level, const char *s)
@@ -108,17 +164,13 @@ static void dologpost(const void *object, const int level, const char *s)
     // what about sys_printhook_verbose ?
     if (STUFF->st_printhook)
     {
-        snprintf(upbuf, MAXPDSTRING-1, "verbose(%d): %s", level, s);
+        pd_snprintf(upbuf, MAXPDSTRING-1, "verbose(%d): %s", level, s);
         (*STUFF->st_printhook)(upbuf);
     }
-    else if (sys_printtostderr)
+    else if (sys_printtostderr || !sys_havetkproc())
     {
 #ifdef _WIN32
-    #ifdef _MSC_VER
-        fwprintf(stderr, L"verbose(%d): %S", level, s);
-    #else
-        fwprintf(stderr, L"verbose(%d): %s", level, s);
-    #endif
+        fwprintf(stderr, L"verbose(%d): " PD_FWPRINTF_NARROW_FORMATTER, level, s);
         fflush(stderr);
 #else
         fprintf(stderr, "verbose(%d): %s", level, s);
@@ -135,7 +187,7 @@ void logpost(const void *object, int level, const char *fmt, ...)
     va_list ap;
     if (level > PD_DEBUG && !sys_verbose) return;
     va_start(ap, fmt);
-    vsnprintf(buf, MAXPDSTRING-1, fmt, ap);
+    pd_vsnprintf(buf, MAXPDSTRING-1, fmt, ap);
     va_end(ap);
     strcat(buf, "\n");
 
@@ -148,7 +200,7 @@ void startlogpost(const void *object, const int level, const char *fmt, ...)
     va_list ap;
     if (level > PD_DEBUG && !sys_verbose) return;
     va_start(ap, fmt);
-    vsnprintf(buf, MAXPDSTRING-1, fmt, ap);
+    pd_vsnprintf(buf, MAXPDSTRING-1, fmt, ap);
     va_end(ap);
 
     dologpost(object, level, buf);
@@ -162,7 +214,7 @@ void post(const char *fmt, ...)
     t_int arg[8];
     int i;
     va_start(ap, fmt);
-    vsnprintf(buf, MAXPDSTRING-1, fmt, ap);
+    pd_vsnprintf(buf, MAXPDSTRING-1, fmt, ap);
     va_end(ap);
     strcat(buf, "\n");
 
@@ -176,7 +228,7 @@ void startpost(const char *fmt, ...)
     t_int arg[8];
     int i;
     va_start(ap, fmt);
-    vsnprintf(buf, MAXPDSTRING-1, fmt, ap);
+    pd_vsnprintf(buf, MAXPDSTRING-1, fmt, ap);
     va_end(ap);
 
     dopost(buf);
@@ -229,7 +281,7 @@ EXTERN void error(const char *fmt, ...)
     int i;
 
     va_start(ap, fmt);
-    vsnprintf(buf, MAXPDSTRING-1, fmt, ap);
+    pd_vsnprintf(buf, MAXPDSTRING-1, fmt, ap);
     va_end(ap);
     strcat(buf, "\n");
 
@@ -246,7 +298,7 @@ void verbose(int level, const char *fmt, ...)
     if (level > sys_verbose) return;
 
     va_start(ap, fmt);
-    vsnprintf(buf, MAXPDSTRING-1, fmt, ap);
+    pd_vsnprintf(buf, MAXPDSTRING-1, fmt, ap);
     va_end(ap);
     strcat(buf, "\n");
 
@@ -272,20 +324,21 @@ void pd_error(const void *object, const char *fmt, ...)
     static int saidit = 0;
 
     va_start(ap, fmt);
-    vsnprintf(buf, MAXPDSTRING-1, fmt, ap);
+    pd_vsnprintf(buf, MAXPDSTRING-1, fmt, ap);
     va_end(ap);
     strcat(buf, "\n");
 
     doerror(object, buf);
 
-    error_object = object;
+    if(object)
+        error_object = object;
     strncpy(error_string, buf, 256);
     error_string[255] = 0;
 
     if (object && !saidit)
     {
-        if (sys_havegui())
-            logpost(NULL, 4,
+        if (sys_havetkproc())
+            logpost(NULL, PD_VERBOSE,
                 "... you might be able to track this down from the Find menu.");
         saidit = 1;
     }
@@ -308,12 +361,15 @@ void glob_findinstance(t_pd *dummy, t_symbol*s)
     // revert s to (potential) pointer to object
     PD_LONGINTTYPE obj = 0;
     const char*addr;
+    int result = 0;
     if(!s || !s->s_name)
         return;
     addr = s->s_name;
-    if (('.' != addr[0]) && ('0' != addr[0]))
-        return;
-    if (!sscanf(addr+1, "x%lx", &obj))
+    if (!result)
+        result = sscanf(addr, PDGUI_FORMAT__OBJECT, &obj);
+    if (!result && (('.' == addr[0]) || ('0' == addr[0])))
+        result = sscanf(addr+1, "x%lx", &obj);
+    if (!result)
         return;
 
     if(!obj)
@@ -329,7 +385,7 @@ void bug(const char *fmt, ...)
     t_int arg[8];
     int i;
     va_start(ap, fmt);
-    vsnprintf(buf, MAXPDSTRING-1, fmt, ap);
+    pd_vsnprintf(buf, MAXPDSTRING-1, fmt, ap);
     va_end(ap);
 
     pd_error(0, "consistency check failed: %s", buf);
