@@ -573,17 +573,8 @@ proc ::pd_menus::forgetpreferences {} {
 
 proc ::pd_menus::create_preferences_menu {mymenu} {
     menu $mymenu
-    $mymenu add command -label [_ "Path..."] \
-        -command {::pd_menucommands::scheduleAction pdsend "pd start-path-dialog"}
-    $mymenu add command -label [_ "Startup..."] \
-        -command {::pd_menucommands::scheduleAction pdsend "pd start-startup-dialog"}
-    $mymenu add command -label [_ "Audio..."] \
-        -command {::pd_menucommands::scheduleAction pdsend "pd audio-properties"}
-    $mymenu add command -label [_ "MIDI..."] \
-        -command {::pd_menucommands::scheduleAction pdsend "pd midi-properties"}
-    $mymenu add check -label [_ "Zoom New Windows"] \
-        -variable ::zoom_open \
-        -command {::pd_menucommands::scheduleAction pdsend "pd zoom-open $zoom_open"}
+    $mymenu add command -label [_ "Edit Preferences..."] \
+        -command {menu_preference_dialog}
     $mymenu add  separator
     $mymenu add command -label [_ "Save All Preferences"] \
         -command {::pd_menucommands::scheduleAction pdsend "pd save-preferences"}
@@ -593,6 +584,10 @@ proc ::pd_menus::create_preferences_menu {mymenu} {
         -command {::pd_menucommands::scheduleAction ::pd_menus::loadpreferences}
     $mymenu add command -label [_ "Forget All..."] \
         -command {::pd_menucommands::scheduleAction ::pd_menus::forgetpreferences}
+    $mymenu add  separator
+    $mymenu add check -label [_ "Tabbed preferences"] \
+        -variable ::dialog_preferences::use_ttknotebook \
+        -command {::pd_menucommands::scheduleAction ::dialog_preferences::write_usettknotebook}
 }
 
 # ------------------------------------------------------------------------------
